<?php
/*
Plugin Name: Tag Manager
plugin URI: https://99robots.com/products/wp-tag-manager/
Description: Limit the number of tags a user can add to a post, and the number of tags displayed in a post.
version: 3.0.0
Author: 99 Robots
Author URI: https://99robots.com/
License: GPL2
*/

/**
 * Global Definitions
 */

/* Item Name */

if (!defined('WPSITE_LIMIT_TAGS_ITEM_NAME'))
    define('WPSITE_LIMIT_TAGS_ITEM_NAME', 'WP Tag Manager');

/* Store URL */

if (!defined('WPSITE_LIMIT_TAGS_STORE_URL'))
    define('WPSITE_LIMIT_TAGS_STORE_URL', 'https://99robots.com');

/* Store URL */

if (!defined('WPSITE_LIMIT_TAGS_MIN_PHP_VERSION'))
    define('WPSITE_LIMIT_TAGS_MIN_PHP_VERSION', '5.4.0');

/* Plugin Name */

if (!defined('WPSITE_LIMIT_TAGS_PLUGIN_NAME'))
    define('WPSITE_LIMIT_TAGS_PLUGIN_NAME', trim(dirname(plugin_basename(__FILE__)), '/'));

/* Plugin directory */

if (!defined('WPSITE_LIMIT_TAGS_PLUGIN_DIR'))
    define('WPSITE_LIMIT_TAGS_PLUGIN_DIR', WP_PLUGIN_DIR . '/' . WPSITE_LIMIT_TAGS_PLUGIN_NAME);

/* Plugin url */

if (!defined('WPSITE_LIMIT_TAGS_PLUGIN_URL'))
    define('WPSITE_LIMIT_TAGS_PLUGIN_URL', WP_PLUGIN_URL . '/' . WPSITE_LIMIT_TAGS_PLUGIN_NAME);

/* Plugin verison */

if (!defined('WPSITE_LIMIT_TAGS_VERSION_NUM'))
    define('WPSITE_LIMIT_TAGS_VERSION_NUM', '3.0.0');


/**
 * Activatation / Deactivation
 */

register_activation_hook( __FILE__, array('WPSiteLimitTags', 'register_activation'));

/**
 * Hooks / Filter
 */

add_action('init', array('WPSiteLimitTags', 'load_textdoamin'));
add_action('admin_menu', array('WPSiteLimitTags', 'wpsite_limit_tags_menu_page'));
add_action('admin_enqueue_scripts', array('WPSiteLimitTags', 'wpsite_limit_tags_include_admin_scripts'));
add_filter('widget_tag_cloud_args', array('WPSiteLimitTags', 'wpsite_limit_tags_set_number'));
add_filter('term_links-post_tag',array('WPSiteLimitTags', 'wpsite_limit_tags_post'));

// Add Settings Link

$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", array('WPSiteLimitTags', 'wpsite_limit_tags_settings_link'));

// Add auto-detection for updates

$path = plugin_basename( __FILE__ );
add_action("after_plugin_row_{$path}", array('WPSiteLimitTags', 'in_plugin_update_message'), 10, 3 );

/**
 * AJAX
 */

add_action('wp_ajax_wpsite_add_tag', array('WPSiteLimitTags', 'add_tag'));
add_action('wp_ajax_wpsite_delete_tag', array('WPSiteLimitTags', 'delete_tag'));

// Software Licensing Updates

add_action('wp_ajax_wps_lt_license_action', array('WPSiteLimitTags', 'license_action'));

if ( !class_exists('EDD_SL_Plugin_Updater') ) {
	include_once( dirname( __FILE__ ) . '/include/edd-software-licensing-updates/EDD_SL_Plugin_Updater.php' );
}

$edd_updater = new EDD_SL_Plugin_Updater( WPSITE_LIMIT_TAGS_STORE_URL , __FILE__, array(
		'version' 	=> WPSITE_LIMIT_TAGS_VERSION_NUM,
		'license' 	=> WPSiteLimitTags::get_license_key(),
		'item_name' => WPSITE_LIMIT_TAGS_ITEM_NAME,
		'author' 	=> '99 Robots',
	)
);

/**
 *  WPSiteLimitTags main class
 *
 * @since 1.0.0
 */

class WPSiteLimitTags {

	/**
	 * text_domain
	 *
	 * (default value: 'wpsite-limit-tags')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $text_domain = 'wpsite-limit-tags';

	/**
	 * prefix
	 *
	 * (default value: 'wpsite_limit_tags_')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $prefix = 'wpsite_limit_tags_';

	/**
	 * prefix_dash
	 *
	 * (default value: 'wps-lt-')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $prefix_dash = 'wps-lt-';

	/**
	 * settings_page
	 *
	 * (default value: 'wpsite-tag-manager-admin-menu-settings')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $settings_page = 'wpsite-tag-manager-admin-menu-settings';

	/**
	 * usage_page
	 *
	 * (default value: 'wpsite-tag-manager-admin-menu-usage')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $usage_page = 'wpsite-tag-manager-admin-menu-usage';

	/**
	 * default
	 *
	 * @var mixed
	 * @access private
	 * @static
	 */
	private static $default = array(
		'tags'		=> array(),
		'settings'	=> array(
			'num_tags'				=> 10, // 1-20
			'max_display_tags'		=> 5, // 1-20
			'meta_box'				=> array(),
			'cloud_num_displayed'	=> 45, // 1-45
			'cloud_largest_font'	=> 22, // 8-40 even
			'cloud_smallest_font'	=> 8 // 8-40 even
		)
	);

	/**
	 * max_tags
	 *
	 * (default value: 50)
	 *
	 * @var int
	 * @access private
	 * @static
	 */
	private static $max_tags = 50;

	/**
	 * max_display_tags_post
	 *
	 * (default value: 5)
	 *
	 * @var int
	 * @access private
	 * @static
	 */
	private static $max_display_tags_post = 5;

	/**
	 * max_display_tags
	 *
	 * (default value: 20)
	 *
	 * @var int
	 * @access private
	 * @static
	 */
	private static $max_display_tags = 20;

	/**
	 * largest_display_tags
	 *
	 * (default value: 25)
	 *
	 * @var int
	 * @access private
	 * @static
	 */
	private static $largest_display_tags = 25;

	/**
	 * plugin_name
	 *
	 * (default value: 'WP Tag Manager')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $plugin_name = 'WP Tag Manager';

	/**
	 * plugin_update_endpoint
	 *
	 * (default value: 'http://versiondata.com/wpsite-tag-manager.json')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $plugin_update_endpoint = 'http://versiondata.com/wpsite-tag-manager.json';

	/**
	 * Load the text domain
	 *
	 * @since 1.0.0
	 */
	static function load_textdoamin() {
		load_plugin_textdomain(self::$text_domain, false, WPSITE_LIMIT_TAGS_PLUGIN_DIR . '/languages');
		require_once('wpsite_add_post_meta_box.php');
	}

	/**
	 * Hooks to 'init'
	 *
	 * @since 1.0.0
	 */
	static function register_activation() {

		/* Check if multisite, if so then save as site option */

		if (is_multisite()) {
			add_site_option('wpsite_limit_tags_version', WPSITE_LIMIT_TAGS_VERSION_NUM);
		} else {
			add_option('wpsite_limit_tags_version', WPSITE_LIMIT_TAGS_VERSION_NUM);
		}

		// Remove all tag whitespace and invalid characters from tags

		$settings = get_option('wpsite_limit_tags_settings');

		if ($settings === false || !is_array($settings))
			$settings = self::$default;

		$new_tags = array();

		foreach ($settings['tags'] as $tag => $val) {
			$tag = stripcslashes(sanitize_text_field($tag));
			$tag = str_replace(' ', '-', $tag);
			$tag = preg_replace('/[^A-Za-z0-9-]/', '', $tag);
			$tag = strtolower($tag);

			$new_tags[$tag] = false;
		}

		$settings['tags'] = $new_tags;

		update_option('wpsite_limit_tags_settings', $settings);
	}

	/**
	 * Perform all the actions with a license
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function license_action() {

		if ( !isset($_POST['license_key']) ) {
			echo '';
			die();
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> $_POST['license_action'],
			'license' 		=> $_POST['license_key'],
			'item_name' 	=> urlencode( WPSITE_LIMIT_TAGS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( WPSITE_LIMIT_TAGS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			echo $response;
			die();
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( $_POST['license_action'] == 'activate_license' ) {
			self::update_license_key($_POST['license_key']);
		} else if ( $_POST['license_action'] == 'deactivate_license' ) {
			self::update_license_key('');
		}

		// decode the license data

		echo $license_data->license;

		die();

	}

	/**
	 * Check if license key is active
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function is_license_active( $license_key ) {

		if ( !isset( $license_key ) ) {
			return false;
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> 'check_license',
			'license' 		=> $license_key,
			'item_name' 	=> urlencode( WPSITE_LIMIT_TAGS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( WPSITE_LIMIT_TAGS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// decode the license data

		if ( isset($license_data->license) && $license_data->license == 'valid' ) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Get the license key
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function get_license_key() {

		// Get the license key from based on WordPress install

		if ( function_exists('is_multisite') && is_multisite() ) {
			$license_key = get_site_option(self::$prefix . 'license_key');
		} else {
			$license_key = get_option(self::$prefix . 'license_key');
		}

		// Check if license key is set

		if ( $license_key === false ) {
			$license_key = '';
		}

		return $license_key;
	}

	/**
	 * Update the license key
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function update_license_key($license_key) {

		if ( isset( $license_key ) ) {

			// Get the license key from based on WordPress install

			if ( function_exists('is_multisite') && is_multisite() ) {

				update_site_option(self::$prefix . 'license_key', $license_key);

			} else {

				update_option(self::$prefix . 'license_key', $license_key);

			}
		}
	}

	/**
	 * Hooks to 'admin_plugin_row{$plugin_file}'
	 *
	 * @access public
	 * @param mixed $plugin_data
	 * @return void
	 */
	static function in_plugin_update_message($plugin_file, $plugin_data, $status) {

		$version_data = get_transient(self::$prefix . 'version_data');

		if ($version_data === false) {
			$ch = curl_init();
			curl_setopt ($ch, CURLOPT_URL, self::$plugin_update_endpoint);
			curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
			$version_data = curl_exec($ch);
			curl_close($ch);
			set_transient(self::$prefix . 'version_data', $version_data, 60 * 60);
		}

		$version_data = json_decode($version_data);

		if (version_compare(WPSITE_LIMIT_TAGS_VERSION_NUM, $version_data->version, '<')) {
			$message = __('There is a new version of ' . self::$plugin_name . ' available. Version ' . $version_data->version . ' is ready. ', self::$text_domain);
			$message .= ' <a href="' . $version_data->download_url . '" target="_blank">' . __( 'Download Now', self::$text_domain). '</a>';
			$message .= '.';

			echo '</tr><tr class="plugin-update-tr"><td colspan="3" class="plugin-update"><div class="update-message">' . $message . '</div></td>';
			?><script type="text/javascript">
				jQuery(document).ready(function($){
					$("#wpsite-tag-manager").addClass('update');
				});
			</script><?php
		}
	}

	/**
	 * Hooks to 'plugin_action_links_' filter
	 *
	 * @since 1.0.0
	 */
	static function wpsite_limit_tags_settings_link($links) {
		$settings_link = '<a href="options-general.php?page=' . self::$settings_page . '">Settings</a>';
		array_unshift($links, $settings_link);
		return $links;
	}


	/**
	 * Hooks to 'widget_tag_cloud_args' filter
	 *
	 * @since 1.0.0
	 */
	static function wpsite_limit_tags_set_number($args) {

		$settings = get_option('wpsite_limit_tags_settings');

		/* Default values */

		if ($settings === false || !is_array($settings))
			$settings = self::$default;

		$args = array(
			'number'    => (int) $settings['settings']['cloud_num_displayed'],
			'largest'   => (int) $settings['settings']['cloud_largest_font'],
			'smallest'  => (int) $settings['settings']['cloud_smallest_font']
		);
		return $args;
	}

	/**
	 * Hooks to 'term_links-post_tag' filter
	 *
	 * @since 1.0.0
	 */
	static function wpsite_limit_tags_post($terms) {

		$settings = get_option('wpsite_limit_tags_settings');

		/* Default values */

		if ($settings === false || !is_array($settings))
			$settings = self::$default;

		$random_array = $terms;
		shuffle($random_array);

		return array_slice($random_array, 0, (int) $settings['settings']['max_display_tags'], true);
	}

	/**
	 * Hooks to 'admin_menu'
	 *
	 * @since 1.0.0
	 */
	static function wpsite_limit_tags_menu_page() {

	    /* sub menu to the tools menu page */


	    $settings_page_load = add_submenu_page(
	    	'options-general.php', 										// parent slug
	    	__('WPsite Tag Manager', self::$text_domain), 						// Page title
	    	__('WPsite Tag Manager', self::$text_domain), 						// Menu name
	    	'manage_options', 											// Capabilities
	    	self::$settings_page, 										// slug
	    	array('WPSiteLimitTags', 'wpsite_limit_tags_admin_settings')	// Callback function
	    );
	    add_action("admin_print_scripts-$settings_page_load", array('WPSiteLimitTags', 'wpsite_limit_tags_inline_admin_scripts'));
	}

	/**
	 * Hooks to 'admin_print_scripts-$page'
	 *
	 * @since 1.0.0
	 */
	static function wpsite_limit_tags_inline_admin_scripts() {

		// CSS

/*
		wp_register_style('wpsite_limit_tags_admin_css', WPSITE_LIMIT_TAGS_PLUGIN_URL . '/css/wpsite_limit_tags_admin.css');
		wp_enqueue_style('wpsite_limit_tags_admin_css');

		wp_register_style('wpsite_dashboard_css', WPSITE_LIMIT_TAGS_PLUGIN_URL . '/css/wpsite_dashboard.css');
		wp_enqueue_style('wpsite_dashboard_css');
*/

		wp_enqueue_style(self::$prefix . 'bootstrap_css', WPSITE_LIMIT_TAGS_PLUGIN_URL . '/css/nnr-bootstrap.min.css');
		wp_enqueue_style(self::$prefix . 'settings_css', WPSITE_LIMIT_TAGS_PLUGIN_URL . '/css/settings.css');

		// Font Awesome

		wp_enqueue_style('wpsite_limit_tags_fontawesome_css', WPSITE_LIMIT_TAGS_PLUGIN_URL . '/include/fontawesome/css/font-awesome.min.css');

		// Bootstrap

		wp_enqueue_style('wpsite_limit_tags_bootstrap_css', WPSITE_LIMIT_TAGS_PLUGIN_URL . '/include/bootstrap-3.2.0-dist/css/bootstrap.css');
		wp_enqueue_script('wpsite_limit_tags_bootstrap_js', WPSITE_LIMIT_TAGS_PLUGIN_URL . '/include/bootstrap-3.2.0-dist/js/bootstrap.js', array('jquery'));
	}

	/**
	 * Hooks to 'admin_enqueue_scripts'
	 *
	 * @since 1.0.0
	 */
	static function wpsite_limit_tags_include_admin_scripts($hook) {

		if ($hook == 'post.php' || $hook == 'post-new.php' || $hook == 'edit.php') {

			/* CSS */

			wp_register_style('wpsite_limit_tags_admin_css', WPSITE_LIMIT_TAGS_PLUGIN_URL . '/css/wpsite_limit_tags_admin.css');
			wp_enqueue_style('wpsite_limit_tags_admin_css');
		}
	}

	/**
	 * AJAX with action 'wpsite_add_tag'
	 *
	 * @since 1.0.0
	 */
	static function add_tag() {

        $settings = get_option('wpsite_limit_tags_settings');

		/* Default values */

		if ($settings === false || !is_array($settings))
			$settings = self::$default;

		$tags = $settings['tags'];
		$tag = stripcslashes(sanitize_text_field($_POST['wpsite_add_tag']));

		// Validate tag

		$tag = strtolower($tag);
		$tag = str_replace(' ', '-', $tag);
		$tag = str_replace('*', '', $tag);
		$tag = str_replace('[', '', $tag);
		$tag = str_replace(']', '', $tag);
		$tag = str_replace('(', '', $tag);
		$tag = str_replace(')', '', $tag);
		$tag = str_replace("'", '', $tag);
		$tag = str_replace('"', '', $tag);
		$tag = str_replace('\\', '', $tag);
		$tag = str_replace('_', '', $tag);
		$tag = str_replace(',', '', $tag);
		$tag = str_replace('/', '', $tag);
		$tag = preg_replace('/[^\p{L}\p{N}\p{C}\p{S}\p{M}\p{Z}\p{P}-]/', '', $tag);
		$tag = preg_replace("/[{}|;:.+=&^%$#@?!`~]/", '', $tag);

		// Check if tag is already in Tag Universe

		if (count($tags) <= self::$max_tags && !array_key_exists($tag, $tags) && $tag != '') {
			$tags[$tag] = false;
			$settings['tags'] = $tags;

			echo '<span class="wpsite_limit_tag_li" id="wpsite_limit_tag_li_' . $tag . '">
				<a name="delete" id="' . $tag . '" class="wpsite_limit_tags_delete ntdelbutton">X</a>
				' . $tag . '
			</span>';
		}

		global $wp_roles;

		foreach ($wp_roles->role_names as $role_name => $role) {
			$settings['settings']['meta_box'][$role_name] = isset($_POST['wpsite_limit_tags_settings_meta_box_' . $role_name]) && $_POST['wpsite_limit_tags_settings_meta_box_' . $role_name] == 'true' ? true : false;
		}

		$settings['settings']['num_tags'] = sanitize_text_field($_POST['wpsite_limit_tags_settings_num_allowed']);
		$settings['settings']['max_display_tags'] = sanitize_text_field($_POST['wpsite_limit_tags_settings_max_display_tags']);
		$settings['settings']['cloud_num_displayed'] = sanitize_text_field($_POST['wpsite_limit_tags_settings_cloud_num_displayed']);
		$settings['settings']['cloud_largest_font'] = sanitize_text_field($_POST['wpsite_limit_tags_settings_cloud_largest_font']);
		$settings['settings']['cloud_smallest_font'] = sanitize_text_field($_POST['wpsite_limit_tags_settings_cloud_smallest_font']);

		$result = update_option('wpsite_limit_tags_settings', $settings);

		die(); // this is required to return a proper result
	}

	/**
	 * AJAX with action 'wpsite_delete_tag'
	 *
	 * @since 1.0.0
	 */
	static function delete_tag() {

		$settings = get_option('wpsite_limit_tags_settings');

		/* Default values */

		if ($settings === false || !is_array($settings))
			$settings = self::$default;

		unset($settings['tags'][$_POST['wpsite_delete_tag']]);

		update_option('wpsite_limit_tags_settings', $settings);

		echo $_POST['wpsite_delete_tag'];

		die(); // this is required to return a proper result
	}

	/**
	 * Displays the HTML for the 'wpsite-limit-tags-admin-menu-settings' admin page
	 *
	 * @since 1.0.0
	 */
	static function wpsite_limit_tags_admin_settings() {

		$settings = get_option('wpsite_limit_tags_settings');

		/* Default values */

		if ($settings === false)
			$settings = self::$default;

		global $wp_roles;

		global $wp_version;

		$license_key = self::get_license_key();

		require_once('admin/dashboard.php');
	}
}

?>